#include <pthread.h>
#include "sysinfoosd.h"
#include "config.h"
#include "unistd.h"
#ifndef VDRVERSNUM
#include <vdr/config.h>
#endif

cSysInfoOsd::cSysInfoOsd(void) {
	osd = NULL;
	group = -1;
	number = 0;
	lastTime = time_ms();
	extraInfo = false;
	offset = 0;
	lines = 0;
	Present = NULL;
	Following = NULL;
	pArray = NULL;
	currentEvent = 0;
}
 
cSysInfoOsd::~cSysInfoOsd(void) {
	Cancel(0);
	delete osd;
}

eOSState cSysInfoOsd::ProcessKey(eKeys Key) {
	eOSState state = cOsdObject::ProcessKey(Key);
	if (state == osUnknown) {	
		switch (Key) {			
			case kBack:
				return osEnd;
				break;
			default:
				return state;
		};
	}
	return osContinue;
}

void cSysInfoOsd::Action() {
	while(true) {
		ClearBitmap();
		DrawMenu(0,0);
		DisplayBitmap();
		usleep(250000);
	}
}

void cSysInfoOsd::Show() {
	osd=cOsdProvider::NewOsd(config.originx, config.originy);  
    if (osd) {
		tArea Area = { 0, 0, config.width-1, config.height-1, 4};
		osd->SetAreas(&Area, 1);
		SendPalette();
		ClearBitmap();
		DrawMenu(0,0);
		DisplayBitmap();
		Start();
	}     
}

void cSysInfoOsd::SendPalette() {
	if (config.usedxr3 == 0) {
		pal[10]= 0xFF0000CC; // 70% Red
		pal[11]= 0x00FF00CC; // 70% Green
		pal[12]= 0xFFFCFC00;
		pal[13]= 0xFFFCFC00;
		pal[14]= 0xFFFCFC00;
		pal[15]= 0xFFFCFC00;
		// Main window color
		SetColor(1,   config.red,  config.green,   config.blue, config.alpha1);
		// Line color
		SetColor(2,   0,   0,   0, config.alphaborder);
		// Channel name background, Time and titles color
		SetColor(3, 255, 255, 255, config.alpha1);
		// Yellow button
		SetColor(4, 255, 255,   0, config.alpha1);
		// Blue button
		SetColor(5,   0,   0, 255, config.alpha1);
		// Green button color
		SetColor(6,   0, 255,   0, config.alpha1);
		// Dark Yellow
		SetColor(7,  50,  50,   0, config.alpha1);
		// Darker color for second half background
		SetColor(8,   (config.red>50) ? config.red-50 : 0, 
			(config.green>50) ? config.green-50 : 0,
			(config.blue>50) ? config.blue-50 : 0,
			config.alpha1);
		// Subtitle text
		SetColor(9, 255, 255, 0, config.alpha1); 
	} else {
		// DXR3 workaround
		pal[0]= clrTransparent; 
		// Main window color
		//    pal[1]= clrBackground; 
		// Line color
		pal[2]= clrBlack; 
		// Channel name background, Time and titles color
		pal[3]= clrWhite; 
		// Yellow button
		pal[4]= clrYellow; 
		// Blue button
		pal[5]= clrBlue; 
		// Green button color
		pal[6]= clrGreen; 
		// Black
		pal[7]= clrBlack; 
		// Darker color for second half background
		pal[8]= clrBlack; 
		// Subtitle text
		pal[9]= clrRed; 
		// Red button color
		pal[10]= clrRed;
		pal[11]= 0xFFFCFC00;
		pal[12]= 0xFFFCFC00;
		pal[13]= 0xFFFCFC00;
		pal[14]= 0xFFFCFC00;
		pal[15]= 0xFFFCFC00;
	}
}

void cSysInfoOsd::SetColor(int Index, int Red, int Green, int Blue, int Alpha) {
  if ((Index >= 0) && (Index <= 15) && (Red >= 0) && (Red <= 255) && (Blue >= 0) && (Blue <= 255) && (Green >= 0) && (Green <= 255)&& (Alpha >= 0) && (Alpha <= 255)) {
    pal[Index] = 0;
    pal[Index] += Alpha;
    pal[Index] <<= 8;
    pal[Index] += Blue;
    pal[Index] <<= 8;
    pal[Index] += Green;
    pal[Index] <<= 8;
    pal[Index] += Red;
  }
}

void cSysInfoOsd::ClearBitmap() {
	osd->DrawRectangle(0, 0, config.width, config.height, pal[0]);
}

void cSysInfoOsd::DrawMenu(int delta, int highlight) {
    ClearBitmap();
    int height = config.height;
    int width = config.width;
    // Draw main window
    osd->DrawRectangle(0, 0, width, height, pal[1]);
    osd->DrawRectangle(0, 0, width, LINEHEIGHT+3, pal[3]);
    for(int i=0; i<width; i++) {
		osd->DrawPixel(i, 0, pal[2]);
		osd->DrawPixel(i, 1, pal[2]);
		osd->DrawPixel(i, LINEHEIGHT+5, pal[2]);
		osd->DrawPixel(i, LINEHEIGHT+6, pal[2]);	
		osd->DrawPixel(i, height-2, pal[2]);
		osd->DrawPixel(i, height-1, pal[2]);	
		osd->DrawPixel(i, 104, pal[2]);
		osd->DrawPixel(i, 105, pal[2]);
    }
    for(int i=0; i<height; i++) {
		osd->DrawPixel(0, i, pal[2]);
		osd->DrawPixel(1, i, pal[2]);
		osd->DrawPixel(width-2, i, pal[2]);
		osd->DrawPixel(width-1, i, pal[2]);
		if(i>104) {
			osd->DrawPixel(width/2, i, pal[2]);
			osd->DrawPixel((width/2)+1, i, pal[2]);	
		}	
		if(i<LINEHEIGHT+5) {
			osd->DrawPixel(490, i, pal[2]);
			osd->DrawPixel(491, i, pal[2]);	
		}
    }

	osd->DrawText(10, 3, "System information", pal[2],pal[3],font);
	osd->DrawText(500, 3, "by Kikko77", pal[2],pal[3],font);
    //Draw Body
    osd->DrawText(15, 40, "Linux Kernel:", pal[3],pal[1],font);
    osd->DrawText(15, 70, "Cpu Type:", pal[3],pal[1],font);
	osd->DrawText(15, 107, "Cpu:", pal[3],pal[1],font);
	osd->DrawText(((config.width/2)+15), 107, "Memory:", pal[3],pal[1],font);
	
    DisplayData();
}

void cSysInfoOsd::DisplayData(void) {
	int iPerCentCpu=0;
	int iPerCentMem=0;	
	int iTopGraph=137;
	int iDownGraph=240;
	char cTemp[100];
	double dTemp=0;
	double d=0;
	int i=0;
	
	// Kernel e cpu type
	ExecShellCmd("cat /proc/version | cut -c 15-21", cTemp);
	osd->DrawText(160, 40, cTemp, pal[3],pal[1],font);
	ExecShellCmd("cat //proc//cpuinfo | grep 'model name' | cut -c 14-58", cTemp);
	osd->DrawText(160, 70, cTemp, pal[3],pal[1],font);

	// Cpu Speed
	osd->DrawText(30, 137, "Speed:", pal[3],pal[1],font);
	ExecShellCmd("cat /proc/cpuinfo | grep 'MHz' | cut -c 12-60", cTemp);
	strcpy(cTemp, itoa(CharToInt(cTemp)));
	strcat(cTemp, "Mhz");
	osd->DrawText(110, 137, cTemp, pal[3],pal[1],font);
	// Calculate cpu free
	ExecShellCmd("CPU=0; for I in `ps -eo ""%C"" | grep ""\\.""`; do CPU=`echo $CPU $I | awk '{ print $1 + $2 }'`; done; echo $CPU", cTemp);
	iPerCentCpu=CharToInt(cTemp);
	if (iPerCentCpu>100) iPerCentCpu=100;
	osd->DrawText(30, 167, "Used:", pal[3],pal[1],font);
	strcpy(cTemp, (itoa(iPerCentCpu)));
	strcat(cTemp, "%");
	osd->DrawText(110, 167, cTemp, pal[3],pal[1],font);
	osd->DrawText(30, 197, "Free:", pal[3],pal[1],font);
	strcpy(cTemp, itoa(100-iPerCentCpu));
	strcat(cTemp, "%");	
	osd->DrawText(110, 197, cTemp, pal[3],pal[1],font);
	// Cpu Graph
    osd->DrawRectangle(220, iTopGraph, 290, iDownGraph, pal[2]);
	d=((iDownGraph-iTopGraph)/100)*iPerCentCpu;
	i=(int)d;
	strcpy(cTemp, itoa(i));
	strcat(cTemp, "%");
	osd->DrawRectangle(224, iDownGraph-i, 286, iDownGraph-4, pal[5]);
	osd->DrawText(232, 177, cTemp, pal[3],pal[1],font);
	
	// total memory
	osd->DrawText(((config.width/2)+30), 137, "Total:", pal[3],pal[1],font);
	ExecShellCmd("cat /proc/meminfo | grep -i 'MEMTOTAL' | cut -c 16-23 | tr -d ' '", cTemp);
	dTemp=atof(cTemp);
	int iMemTot=(int)(dTemp/1000);
	strcpy(cTemp, itoa(iMemTot));
	strcat(cTemp, "Mb");
	osd->DrawText(((config.width/2)+130), 137, cTemp, pal[3],pal[1],font);
	// Free memory
	osd->DrawText(((config.width/2)+30), 197, "Free:", pal[3],pal[1],font);
	ExecShellCmd("cat /proc/meminfo | grep -i 'MEMFREE' | cut -c 16-23 | tr -d ' '", cTemp);
	dTemp=atof(cTemp);
	int iMemFree=(int)(dTemp/1000);
	strcpy(cTemp, itoa(iMemFree));
	strcat(cTemp, "Mb");
	osd->DrawText(((config.width/2)+130), 197, cTemp, pal[3],pal[1],font);
	// Used memory
	osd->DrawText(((config.width/2)+30), 167, "Used:", pal[3],pal[1],font);
	int iMemUsed=iMemTot-iMemFree;
	strcpy(cTemp, itoa(iMemUsed));
	strcat(cTemp, "Mb");
	osd->DrawText(((config.width/2)+130), 167, cTemp, pal[3],pal[1],font);
	// Mem Graph
	iPerCentMem=(100*iMemUsed)/iMemTot;
    osd->DrawRectangle(((config.width/2)+220), iTopGraph, ((config.width/2)+290), iDownGraph, pal[2]);
	d=((iDownGraph-iTopGraph)/100)*iPerCentMem;
	i=(int)d;
	strcpy(cTemp, itoa(i));
	strcat(cTemp, "%");
	osd->DrawRectangle(((config.width/2)+224), iDownGraph-i, ((config.width/2)+286), iDownGraph-4, pal[5]);
	osd->DrawText(((config.width/2)+232), 177, cTemp, pal[3],pal[1],font);
	
}

int cSysInfoOsd::CharToInt(char *cInput) {
	double dTemp=0;
	int iReturn=0;
	dTemp=atof(cInput);
	iReturn=(int)dTemp;	
	return iReturn;
}

void cSysInfoOsd::ExecShellCmd(char *cCommand, char *cReturn) {
	for (int i=0; i<100; i++) cReturn[i]=' ';
	FILE *infile=popen(cCommand,"r");
	fread(cReturn, 1, 100, infile);
	fclose(infile);
	for (int i=0; i<100; i++) {
		if ((cReturn[i]==' ') && (cReturn[i+1]==' ')) cReturn[i]='\0';
	}
}

void cSysInfoOsd::DisplayBitmap() {
  osd->Flush();
}